from ..shared.client.client import Client
from .model import Model


def dto(m: list) -> Model:
    return Model(
        id=m.get('id'),
        name=m.get('name'),
        version=m.get('version'),
        description=m.get('description'),
        args=m.get('args'),
        video=m.get('video'),
        example_outputs=m.get('example_outputs'),
        alias=m.get('alias'),
    )


class Models(Client):
    def __init__(self, api_key: str,mm_url: str) -> None:
        super().__init__(api_key,mm_url)
    
    def list(self):
        """
        list - list all models
        """
        models = super().get("/api/v1/models")

        result = []
        for m in models:
            parsed = dto(m)
            result.append(parsed)
        return result

    def get_by_id(self, id: str):
        """
        get_by_id - get a model by id
        """
        model = super().get("/api/v1/models/%s" % id)
        return dto(model)

    def get_by_name(self, name: str):
        """
        get_by_name - get a model by name
        """
        try:
            path = "/api/v1/models/name/%s" % name
            model = super().get(path)
            return dto(model)
        except Exception as e:
            raise Exception('error getting model %s' % e)



from .deployments import Deployments
from .model import CreateDeployment

import pytest
import os


_API_KEY = os.getenv("X_MEDIAMAGIC_KEY")


def test_can_create_deployment():
    deployment_client = Deployments(api_key=_API_KEY)
    deployment = deployment_client.create(deployment=CreateDeployment(
        name="test-deployment",
        model="pingponai/background-removal",
        args={
            'input_image_file': 'https://cdn.mediamagic.dev/media/eb341446-be53-11ed-b4a8-66139910f724.jpg',
        },
    ))
    assert deployment.job.credits_used > 0
    assert deployment.job.id is not None


def test_can_list_deployments():
    deployment_client = Deployments(api_key=_API_KEY)
    deployments = deployment_client.list(0, 10)



from dataclasses import dataclass


@dataclass
class Model:
    id: str
    name: str
    alias: str
    description: str
    example_outputs: list[str]
    args: list[dict]
    video: str
    version: str



from dataclasses import dataclass
from typing import List

@dataclass
class Job:
    id: str
    credits_used: str
    results: List[str]
    status: str
    error: str
    deployment_id: str
    duration: int
    logs: str
    eta: int
    etr: int


@dataclass
class Deployment:
    id: str
    model: str
    job: Job
    status: str
    name: str
    job_id: str
    logs: str

@dataclass
class CreateDeployment:
    model: str
    args: dict
    name: str
    sync: bool = False

    def dict(self):
        return super().dict()

def request_dto(id: str, m: CreateDeployment) -> dict:
    return {
        'model_id': id,
        'args': m.args,
        'name': m.name,
    }

def dto(m: dict) -> Deployment:
    return Deployment(
        id=m.get('id'),
        model=m.get('alias'),
        job=m.get('job'),
        status=m.get('status'),
        logs=m.get('logs'),
        job_id=m.get('job_id'),
        name=m.get('name'),
    )



import requests

class Client:

    api_key: str = ''
    mm_url: str = ''


    def __init__(self, api_key: str,mm_url: str):
        self._api_key = api_key
        self._mm_url = mm_url


    def get(self, path: str):
        try:
            response = requests.get(self._mm_url + path, headers={
                'Content-Type': 'application/json',
                'x-mediamagic-key': self._api_key,
                'Accept': 'application/json',
            })
            response.raise_for_status()
            return response.json()
        except requests.exceptions.HTTPError as e:
            return "A HTTP error occured: %s" % e


    def post(self, path: str, body: dict):
        try:
            response = requests.post(self._mm_url + path, json=body, headers={
                'Content-Type': 'application/json',
                'x-mediamagic-key': self._api_key,
                'Accept': 'application/json',
            })
            response.raise_for_status()
            return response.json()
        except requests.exceptions.HTTPError as e:
            return "A HTTP error occured: %s" % e



import os

from .deployments.deployments import Deployments
from .models.models import Models

_MM_URL = "https://api-qa.mediamagic.ai"

_API_KEY = os.getenv('X_STYLEY_KEY')
_MM_BASE_URL = os.getenv('MM_HOST_URL', _MM_URL)

class Styley:
    """
    Styley - is a vanity class, which simply
    combined the underlying libraries into one
    object with the api configured for each etc.
    """

    models: Models
    deployments: Deployments

    def __init__(self, api_key=_API_KEY, mm_url=_MM_BASE_URL):
        if not api_key:
            raise ValueError("X_STYLEY_KEY missing")
        if not mm_url:
            raise ValueError("MM_BASE_URL missing")
        self.models = Models(api_key=api_key,mm_url=mm_url)
        self.deployments = Deployments(api_key=api_key,mm_url=mm_url, models=self.models)



from .models import Models
import pytest
import os


_API_KEY = os.getenv("X_MEDIAMAGIC_KEY")


def test_can_list_models():
    models_client = Models(_API_KEY)
    models = models_client.list()

    assert len(models) > 0


def test_can_get_model_by_id():
    models_client = Models(_API_KEY)
    models = models_client.list()
    
    model = {}
    for m in models:
        if m.name == 'Magic AI':
            model = m

    m = models_client.get_by_id(id=model.id)

    assert m.name == 'Magic AI'



from ..shared.client.client import Client
from .model import Deployment, dto, request_dto, CreateDeployment,Job
import time
from typing import List

from dataclasses import asdict

COMPLETE = 'complete'
FAILED = 'failed'
ETA = 180

class Deployments(Client):

    def __init__(self, api_key: str,mm_url: str, models) -> None:
        self.models = models
        super().__init__(api_key,mm_url)


    def get_by_id(self, id: str) -> Deployment:
        """
        get_by_id - get a deployment by its ID
        """
        deployment = super().get("/api/v1/deployments/%s" % id)
        return dto(deployment)
    

    def create(self, deployment: CreateDeployment) -> Deployment:
        """
        create - create a new Deployment
        """
        try:
            model = self.models.get_by_id(deployment.model)
            id = model.id
            request = request_dto(id, deployment)
            deployment = super().post("/api/v1/deployments", request)
            response=dto(deployment)
            status=response.status
            eta = ETA
            while status!=COMPLETE and status!=FAILED and eta > 0:
                time.sleep(10)
                job=self.get_job(response.job_id)
                status=job['status']
                eta -= 5
                response.status=status
                if 'logs' in job:
                    response.logs=job['logs']
                response.job=job
            return response
        except Exception as e:
            raise Exception('error creating deployment: %s' % e)


    def list(self) -> List[Deployment]:
        """
        list - list all of your deployments
        """
        deployments = super().get("/api/v1/deployments")
        return deployments
    
    def get_job(self, id:str) -> Job:
        """
        get_job - get job by ID
        """
        path="/api/v1/jobs/"+ id
        job = super().get(path)

        return job

    
